@@ -17,9 +17,9 @@ which provides hooks for hgk to get information. hgk can be found in
 the contrib directory, and the extension is shipped in the hgext
 repository, and needs to be enabled.
 
-The hg view command will launch the hgk Tcl script. For this command
+The :hg:`view` command will launch the hgk Tcl script. For this command
 to work, hgk must be in your search path. Alternately, you can specify
-the path to hgk in your .hgrc file::
+the path to hgk in your configuration file::
 
   [hgk]
   path=/location/of/hgk
@@ -35,7 +35,7 @@ vdiff on hovered and selected revisions.
 '''
 
 import os
-from mercurial import commands, util, patch, revlog, cmdutil
+from mercurial import commands, util, patch, revlog, scmutil
 from mercurial.node import nullid, nullrev, short
 from mercurial.i18n import _
 
@@ -45,7 +45,7 @@ def difftree(ui, repo, node1=None, node2=None, *files, **opts):
         assert node2 is not None
         mmap = repo[node1].manifest()
         mmap2 = repo[node2].manifest()
-        m = cmdutil.match(repo, files)
+        m = scmutil.match(repo[node1], files)
         modified, added, removed  = repo.status(node1, node2, m)[:3]
         empty = short(nullid)
 
@@ -81,7 +81,7 @@ def difftree(ui, repo, node1=None, node2=None, *files, **opts):
         if opts['patch']:
             if opts['pretty']:
                 catcommit(ui, repo, node2, "")
-            m = cmdutil.match(repo, files)
+            m = scmutil.match(repo[node1], files)
             chunks = patch.diff(repo, node1, node2, match=m,
                                 opts=patch.diffopts(ui, {'git': True}))
             for chunk in chunks:
@@ -92,7 +92,7 @@ def difftree(ui, repo, node1=None, node2=None, *files, **opts):
             break
 
 def catcommit(ui, repo, n, prefix, ctx=None):
-    nlprefix = '\n' + prefix;
+    nlprefix = '\n' + prefix
     if ctx is None:
         ctx = repo[n]
     ui.write("tree %s\n" % short(ctx.changeset()[0])) # use ctx.node() instead ??
@@ -135,7 +135,7 @@ def catfile(ui, repo, type=None, r=None, **opts):
     prefix = ""
     if opts['stdin']:
         try:
-            (type, r) = raw_input().split(' ');
+            (type, r) = raw_input().split(' ')
             prefix = "    "
         except EOFError:
             return
@@ -148,12 +148,12 @@ def catfile(ui, repo, type=None, r=None, **opts):
     while r:
         if type != "commit":
             ui.warn(_("aborting hg cat-file only understands commits\n"))
-            return 1;
+            return 1
         n = repo.lookup(r)
         catcommit(ui, repo, n, prefix)
         if opts['stdin']:
             try:
-                (type, r) = raw_input().split(' ');
+                (type, r) = raw_input().split(' ')
             except EOFError:
                 break
         else:
@@ -181,14 +181,14 @@ def revtree(ui, args, repo, full="tree", maxnr=0, parents=False):
                 if i + x >= count:
                     l[chunk - x:] = [0] * (chunk - x)
                     break
-                if full != None:
+                if full is not None:
                     l[x] = repo[i + x]
                     l[x].changeset() # force reading
                 else:
                     l[x] = 1
-            for x in xrange(chunk-1, -1, -1):
+            for x in xrange(chunk - 1, -1, -1):
                 if l[x] != 0:
-                    yield (i + x, full != None and l[x] or None)
+                    yield (i + x, full is not None and l[x] or None)
             if i == 0:
                 break
 
@@ -220,8 +220,8 @@ def revtree(ui, args, repo, full="tree", maxnr=0, parents=False):
 
     # calculate the graph for the supplied commits
     for i, n in enumerate(want_sha1):
-        reachable.append(set());
-        visit = [n];
+        reachable.append(set())
+        visit = [n]
         reachable[i].add(n)
         while visit:
             n = visit.pop(0)
@@ -314,7 +314,8 @@ def view(ui, repo, *etc, **opts):
 cmdtable = {
     "^view":
         (view,
-         [('l', 'limit', '', _('limit number of changes displayed'))],
+         [('l', 'limit', '',
+           _('limit number of changes displayed'), _('NUM'))],
          _('hg view [-l LIMIT] [REVRANGE]')),
     "debug-diff-tree":
         (difftree,
